;===============================================================;
;                                                               ;
; Alien Breed 5                                                 ;
; Visual effect routines (TI-84+CE only)                        ;
;                                                               ;
;===============================================================;

;------------------------------------------------
; fadeIn - fade the screen in
;   input:  none
;   output: none
;------------------------------------------------
fadeIn:
        ld      hl,fadeInSub
        jp      fadeLcd
fadeInSub:
        dec     c
        ret

;------------------------------------------------
; fadeOut - fade the screen out by slowing erasing the palette
;   input:  none
;   output: none
;------------------------------------------------
fadeOut:
        ; save the existing palette
        ld      hl,mpLcdPalette
        ld      de,palBackup
        ld      bc,256*2
        ldir
        ; get started
        ld      hl,fadeOutSub
        jp      fadeLcd
fadeOutSub:
        ld      a,c
        sub     32
        neg
        ld      c,a
        ret

;------------------------------------------------
; fadeOutClear - fade the screen out, then clear vram and restore the palette
;   input:  none
;   output: none
;------------------------------------------------
fadeOutClear:
        call    fadeOut
        call    vramClear
        ld      hl,palBackup
        ld      de,mpLcdPalette
        ld      bc,256*2
        ldir
        ret

;------------------------------------------------
; fadeToGame - fade the screen out, draw the game screen and fade back in
;   input:  none
;   output: none
;------------------------------------------------
fadeToGame:
        call    fadeOut
        call    vramClear
        call    drawTiles
        call    drawObjects
        call    drawAnims
        call    drawEnemies
        call    drawBullets
        call    drawPlayers
        call    showHud
        call    vramFlip
        jp      fadeIn

;------------------------------------------------
; fadeLcd - fade the screen out or in
;   input:  HL => routine to calculate RGB modifier
;   output: none
;------------------------------------------------
fadeLcd:
        ld      (__flSubCalc),hl
        push    iy
        ld      c,32
flOuter:
        ld      b,0                             ; (256)
        ld      iy,mpLcdPalette
        ld      ix,palBackup
flInner:
        push    bc
__flSubCalc             = $+1
        call    $000000
        ld      hl,0
        ; red
        ld      a,(ix+1)
        rrca \ rrca
        and     %00011111
        sub     c
        jr      nc,flSkipR
        xor     a
flSkipR:
        rlca \ rlca
        ld      l,a
        ; green
        ld      e,(ix+1)
        ld      d,(ix)
        sla d \ rl e
        sla d \ rl e
        sla d \ rl e
        ld      a,e
        and     %00011111
        sub     c
        jr      nc,flSkipG
        xor     a
flSkipG:
        ld      d,0
        ld      e,a
        srl e \ rr d
        srl e \ rr d
        srl e \ rr d
        ld      a,l
        or      e
        ld      l,a
        ld      a,h
        or      d
        ld      h,a
        ; blue
        ld      a,(ix)
        and     %00011111
        sub     c
        jr      nc,flSkipB
        xor     a
flSkipB:
        or      h
        ld      h,a
        ld      (iy),h
        ld      (iy+1),l
        inc ix \ inc ix
        inc iy \ inc iy
        pop     bc
        djnz    flInner
        push    bc
        ld      bc,FADE_DELAY
        call    delayBC
        pop     bc
        dec     c
        jr      nz,flOuter
        pop     iy
        ret

;------------------------------------------------
; updateCursorFlash - update the last entry of mpLcdPalette for the flashing cursor/text
;   input:  A = 0-15
;   output: none
;------------------------------------------------
updateCursorFlash:
        and     $0F
        ld      l,a
        ld      h,2
        mlt     hl
        ld      de,palTextFlash
        add     hl,de
        ld      de,(hl)
        ld      ix,mpLcdPalette+512-2
        ld      (ix),e
        ld      (ix+1),d
        ret

;------------------------------------------------
; iniCountdownPalette - initialise change of palette to red hue
;   input:  none
;   output: none
;------------------------------------------------
iniCountdownPalette:
        ld      a,INI_PAL_SHIFT
        ld      (palShift),a
        ret

;------------------------------------------------
; fullCountdownPalette - fully update the palette to the red hue (when loading game state)
;   input:  none
;   output: none
;------------------------------------------------
fullCountdownPalette:
        call    iniCountdownPalette
fcpLoop:
        call    updateCountdownPalette
        ld      a,(palShift)
        or      a
        ret     z
        jr      fcpLoop

;------------------------------------------------
; updateCountdownPalette - continue transitioning the palette change for the countdown sequence
;   input:  none
;   output: none
;------------------------------------------------
updateCountdownPalette:
        ld      a,(palShift)
        or      a
        ret     z
        dec     a
        ld      (palShift),a
        ld      ix,mpLcdPalette+4
        ld      b,29
ucpLoop:
        ; green
        ld      e,(ix+1)
        ld      d,(ix)
        sla d \ rl e
        sla d \ rl e
        sla d \ rl e
        ld      a,e
        and     %00011111
        jr      z,ucpBlue
        dec     a
        ld      d,0
        ld      e,a
        srl e \ rr d
        srl e \ rr d
        srl e \ rr d
        ld      a,(ix+1)
        and     %11111100
        or      e
        ld      (ix+1),a
        ld      a,(ix)
        and     %00011111
        or      d
        ld      (ix),a
ucpBlue:
        ; blue
        ld      a,(ix)
        and     %00011111
        jr      z,ucpNext
        dec     a
        ld      e,a
        ld      a,(ix)
        and     %11100000
        or      e
        ld      (ix),a
ucpNext:
        inc     ix
        inc     ix
        djnz    ucpLoop
        ret

;------------------------------------------------
; levelExplode - do multiple explosion animations and then game is over
;   input:  none
;   output: none
;------------------------------------------------
timerZero:
        pop     hl                              ; pop the call
levelExplode:
        ; do the level exploding animation, which comprises of the following:
        ; - shake the screen vertically
        ; - fade out
        ; - fade back in with the level without sprites
        ; - do lots of explosion animations
        ; - go to game over screen
shakeScreen:
        ld      b,15                            ; B = shake screen 15 times
shakeScreenLoop:
        push    bc
        call    scrollDown
        call    scrollDown
        call    shakeDraw
        ld      b,6
        call    delay
        call    scrollUp
        call    scrollUp
        call    shakeDraw
        ld      b,6
        call    delay
        call    keyScan
        ld      a,(kbdG2)
        bit     kbitAlpha,a
        pop     bc
        jp      nz,gameOver
        djnz    shakeScreenLoop
        ; neither player is now active
        xor     a
        ld      (p1p),a
        ld      (p2p),a
        call    fadeOut                         ; fade screen out
        ld      b,20
        call    delay                           ; wait a bit
        call    vramClear
        call    drawTiles
        call    vramFlip
        call    fadeIn
        call    clearAnimTable                  ; clear animTable
        ld      b,50                            ; B = 50 frames to loop showing explosions
levelExplosionLoop:
        push    bc                              ; save frame counter
        call    updateAnims
        call    randomExplosion
        call    drawTiles
        call    drawAnims
        call    vramFlip
        ld      b,6
        call    delay
        call    keyScan
        ld      a,(kbdG2)
        bit     kbitAlpha,a
        pop     bc                              ; restore frame counter
        jp      nz,gameOver
        djnz    levelExplosionLoop              ; loop
        call    fadeOutClear
        jp      gameOver
shakeDraw:
        call    drawTiles
        call    drawObjects
        call    drawAnims
        call    drawEnemies
        call    drawBullets
        call    drawPlayers
        call    showHud
        jp      vramFlip

;------------------------------------------------
; doEpilogue - show the epilogue for this episode
;   input:  none
;   output: none
;------------------------------------------------
doEpilogue:
        ; unpack epilogue image
        ld      hl,(pDatFile)
        ld      de,images
        add     hl,de
        ld      de,sbuf
        ld      ix,huffTree
        ld      b,NUMEPISODES+1
        call    HuffExtr

        ; reset vram/vbuf
        ld      hl,vram
        ld      (mpLcdBase),hl
        ld      hl,vbuf2
        ld      (pVbuf),hl

        ld      hl,palEpilogue
        call    draw4bppImage                   ; draw image to vbuf2

        ; scroll it up to vbuf1 (vram)
        ld      b,30                            ; B = 30 frames (8 rows per frame)
splashScroll:
        push    bc
        ld      hl,vram+(320*8)
        ld      de,vram
        ld      bc,320*472
        ldir
        pop     bc
        djnz    splashScroll

        ld      b,240
        call    delay

        ; get pointers to relevant epilogue data
        ld      hl,(episode)
        ld      h,3
        mlt     hl
        ld      de,epilogueTable
        add     hl,de                           ; HL => ptr to epilogue data
        ld      de,(hl)
        ld      hl,(pLvlFile)
        add     hl,de                           ; HL => epilogue data
        ld      de,sbuf
        ld      ix,huffTree
        call    HuffExtr

        ld      hl,sbuf
epilogueLoop:
        ld      a,(hl)                          ; A = pencol position (or 0 for no text this line)
        or      a                               ; check if text line
        jr      nz,epilogueLine
        inc     hl
        ld      b,(hl)                          ; B = # rows to scroll
        inc     hl                              ; HL => next bit data after this scroll
        push    hl
        jr      epilogueScroll
epilogueLine:
        ld      de,0
        ld      e,a
        inc     hl
        push    hl
        pop     ix
        ld      l,0
        call    printString
        push    ix                              ; save pointer to next bit of data
        ld      b,12                             ; B = 12 lines to scroll for a line of text
epilogueScroll:
        push    bc
        ld      hl,vbuf1+320
        ld      de,vbuf1
        ld      bc,320*252
        ldir
        call    keyScan
        ld      a,(kbdG2)
        bit     kbitAlpha,a
        ld      b,8
        call    z,delay
        pop     bc
        djnz    epilogueScroll
        pop     hl
        ld      a,(hl)
        inc     a                               ; check if $FF terminator
        jp      nz,epilogueLoop

        ; scrolling is finished, now print the final screen
        inc     hl                              ; HL => page to print
        push    hl
        call    fadeOutClear
        pop     hl
        jp      displayPageNoClear

.end
